/* GNU FaXile - fax image viewer
 *
 * Copyright (C) 2000 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <wolfgang@ultim.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include "tiffimages.h"
#include "viewer.h"

extern float zoom_factor[];

void refresh_widgets (ViewerData *viewer_data)
{
	int bcounter;

	if (viewer_data->current_page->next == NULL)
		gtk_widget_set_sensitive(viewer_data->bb_buttons[NEXT_PAGE], FALSE);
	else
		gtk_widget_set_sensitive(viewer_data->bb_buttons[NEXT_PAGE], TRUE);
	
	if (viewer_data->current_page->prev == NULL)
		gtk_widget_set_sensitive(viewer_data->bb_buttons[PREV_PAGE], FALSE);
	else
		gtk_widget_set_sensitive(viewer_data->bb_buttons[PREV_PAGE], TRUE);
	
	if (viewer_data->zoom_index == 0)
		gtk_widget_set_sensitive(viewer_data->bb_buttons[ZOOM_OUT], FALSE);
	else {
		gtk_widget_set_sensitive(viewer_data->bb_buttons[ZOOM_OUT], TRUE);
		if (viewer_data->zoom_index == 6)
			gtk_widget_set_sensitive(viewer_data->bb_buttons[ZOOM_IN], FALSE);
		else
			gtk_widget_set_sensitive(viewer_data->bb_buttons[ZOOM_IN], TRUE);
	}

	for (bcounter = 0; 
	     bcounter < viewer_data->fax_file->nbr_images + 1;
	     bcounter++)
		gtk_widget_set_sensitive(viewer_data->th_buttons[bcounter], TRUE);

	gtk_widget_set_sensitive(viewer_data->th_buttons[viewer_data->current_page->nbr], FALSE);

	gtk_widget_queue_resize (viewer_data->page_area->parent);
}

GdkPixmap *pixmap_from_image (GtkWidget *ref_widget,
			      FaxPage *ref_image,
			      int with_frame)
{
	GdkPixmap *new_pixmap;
	GdkBitmap *tmp_bitmap;
	GdkGC *tmp_gc;
	int x, y, width, height;

	width = ref_image->width;
	height = ref_image->height;

	if (with_frame) {
		x = 1;
		y = 1;
		width += 2;
		height += 2;
	} else {
		x = 0;
		y = 0;
	}

	new_pixmap = gdk_pixmap_new (ref_widget->window, width, height, -1);
	tmp_bitmap = gdk_bitmap_create_from_data (ref_widget->window,
						  ref_image->image,
						  ref_image->width,
						  ref_image->height);
	
	gdk_draw_rectangle (new_pixmap,
			    ref_widget->style->white_gc,
			    TRUE,
			    x, y, 
			    ref_image->width,
			    ref_image->height);
	
	tmp_gc = gdk_gc_new (new_pixmap);
	gdk_gc_set_clip_origin (tmp_gc, 0, 0);
	gdk_gc_set_clip_mask (tmp_gc, tmp_bitmap);
	
	gdk_draw_rectangle (new_pixmap,
			    tmp_gc,
			    TRUE,
			    x, y,
			    ref_image->width, ref_image->height);
	
	if (with_frame)
		gdk_draw_rectangle 
			(new_pixmap,
			 ref_widget->style->black_gc,
			 FALSE,
			 0, 0,
			 width - 1, height - 1);

	gdk_pixmap_unref (tmp_bitmap);
	gdk_gc_unref (tmp_gc);
	
	return new_pixmap;
}

void draw_page (ViewerData *viewer_data)
{
	int scr_width, scr_height;
	FaxPage *scr_image;

	if (viewer_data->current_page != NULL) {
		ti_load_fax_page (viewer_data->fax_file,
				 viewer_data->current_page);
		scr_width = (int) (
			(float) viewer_data->current_page->width /
			(float) viewer_data->fax_file->x_res *
			75 * zoom_factor[viewer_data->zoom_index]);
		scr_height = (int) (
			(float) viewer_data->current_page->height /
			(float) viewer_data->fax_file->y_res *
			75 * zoom_factor[viewer_data->zoom_index]);
		scr_image = ti_zoomed_fax_page
			(viewer_data->current_page,
			 scr_width,
			 scr_height,
			 viewer_data->reverse_page);

		ti_unload_fax_page (viewer_data->current_page);

		if (viewer_data->page_pixmap)
			gdk_pixmap_unref (viewer_data->page_pixmap);

		viewer_data->page_pixmap = 
			pixmap_from_image (viewer_data->viewer_window,
					   scr_image, TRUE);

		gtk_drawing_area_size
			(GTK_DRAWING_AREA(viewer_data->page_area), 
			 scr_width + 2, scr_height + 2);

		gdk_window_set_back_pixmap
			(viewer_data->page_area->window,
			 viewer_data->page_pixmap, FALSE);

		refresh_widgets (viewer_data);

		ti_destroy_fax_page (scr_image);

		gtk_widget_unmap (viewer_data->page_area);
		gtk_widget_map (viewer_data->page_area);
	}
}


