/* GNU FaXile - fax image viewer
 *
 * Copyright (C) 2000 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <wolfgang@ultim.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include "tiffimages.h"
#include "viewer.h"

extern float zoom_factor[];
extern void draw_page (ViewerData *viewer_data);
extern GdkPixmap *pixmap_from_image (GtkWidget *ref_widget,
				     FaxPage *ref_image,
				     int with_frame);

inline static int get_thumbed_height (FaxFile *fax_file, FaxPage
				      *image, int width)
{
	float factor;
	int height;

	factor = (float) image->width / (float) width;
	height = image->height / factor;
	height *= (float) fax_file->x_res / (float) fax_file->y_res;

	return height;
}

gint draw_req_page_cb (GtkWidget *widget, DrawReqData *request)
{
	if (request->viewer_data->current_page !=
	    request->requested_page) {
		request->viewer_data->current_page =
			request->requested_page;
		draw_page (request->viewer_data);
	}

	return FALSE;
}

gint destroy_draw_req_cb (GtkWidget *widget, DrawReqData *draw_req_data)
{
	g_free (draw_req_data);

	return FALSE;
}

GtkRcStyle *back_gtkstyle (gushort red, gushort green, gushort blue)
{
	GdkColor color;
	GtkRcStyle *new_style;

	color.red = red;
	color.green = green;
	color.blue = blue;
	new_style = gtk_rc_style_new ();
	new_style->bg[GTK_STATE_NORMAL] = color;
	new_style->color_flags[GTK_STATE_NORMAL] = GTK_RC_BG;

	return new_style;
}

GtkWidget *thumbs_fixed_new (FaxFile *fax_file)
{
	GtkWidget *thumbs_fixed;
	GtkRcStyle *bg_style;
	FaxPage *cur_image;
	guint height;

	height = 10;
	cur_image = fax_file->first;

	while (cur_image != NULL) {
		height += get_thumbed_height (fax_file,
					      cur_image,
					      64)
			+ 24;
		cur_image = cur_image->next;
	}

	bg_style = back_gtkstyle (13824, 12288, 18176);

	thumbs_fixed = gtk_fixed_new ();
	gtk_widget_set_usize (thumbs_fixed, 80, height - 10);
	gtk_widget_modify_style (thumbs_fixed, bg_style);

	gtk_rc_style_unref (bg_style);

	return thumbs_fixed;
}

void add_thumbs (GtkWidget *thumbs_fixed, ViewerData *viewer_data)
{
	GtkWidget *cur_button, *cur_pixmap;
	GtkTooltips *cur_tooltips;
	GtkWidget **b_arr_ptr;
	GdkPixmap *thumb_gdkpix;
	FaxPage *cur_image, *cur_thumb;
	DrawReqData *draw_request;
	gint pos_in_fixed, th_height;
	char *tttext;

	cur_image = viewer_data->fax_file->first;

	viewer_data->th_buttons =
		g_malloc (sizeof (GtkWidget*)
			  *(viewer_data->fax_file->nbr_images + 1));
	b_arr_ptr = viewer_data->th_buttons;
	pos_in_fixed = 10;

	while (cur_image != NULL) {
		ti_load_fax_page (viewer_data->fax_file, cur_image);
		th_height = get_thumbed_height (viewer_data->fax_file,
						cur_image, 64);
		cur_thumb = ti_zoomed_fax_page (cur_image, 64, th_height, FALSE);
		ti_unload_fax_page (cur_image);

		draw_request = g_malloc (sizeof (DrawReqData));

		draw_request->viewer_data = viewer_data;
		draw_request->requested_page = cur_image;

		thumb_gdkpix = pixmap_from_image
			(viewer_data->viewer_window, cur_thumb, TRUE);

		cur_pixmap = gtk_pixmap_new (thumb_gdkpix, NULL);
		cur_button = gtk_button_new ();
		gtk_button_set_relief (GTK_BUTTON(cur_button),
				       GTK_RELIEF_NONE);

		gtk_signal_connect (GTK_OBJECT(cur_button),
				    "clicked",
				    GTK_SIGNAL_FUNC
				    (draw_req_page_cb),
				    draw_request);
		gtk_signal_connect (GTK_OBJECT(cur_button), "destroy",
				    GTK_SIGNAL_FUNC
				    (destroy_draw_req_cb),
				    draw_request);

                gtk_container_add (GTK_CONTAINER(cur_button),
                                   cur_pixmap);

		gtk_fixed_put (GTK_FIXED (thumbs_fixed),
			       cur_button, 5, pos_in_fixed);

		cur_tooltips = gtk_tooltips_new();
		tttext = g_strdup_printf (_("Page %d"), cur_image->nbr + 1);
		gtk_tooltips_set_tip (cur_tooltips,
					cur_button,
					tttext, NULL);
		g_free (tttext);

		gtk_widget_show (cur_pixmap);
		gtk_widget_show (cur_button);

		*b_arr_ptr++ = cur_button;
		cur_image = cur_image->next;
		pos_in_fixed += th_height + 24;
	}
}
